import sys
import argparse
import bdfparser
# Print one horizontal line of a character in the same fashion wscons fonts do
pbmp = lambda a:'/* '+''.join('#' if a & (1 << (7-n)) else '.' for n in range(8))+'*/'

#### Define your unicode ranges here
#[(0,0x4FF),(0x590,0x5FF),(0x16A0,0x16FF),(0x1AB0,0x1AFF),(0x1D00,0x1DFF),(0x2000,0x23FF),(0x2500,0x2BFF)]
ranges = [(0x20,0x500),(0x590,0x600),(0x16A0,0x1700),(0x2000,0x2400),(0x2500,0x2700),(0x27C0,0x2B00)]

# Helper functions to check if within range and correct size
chkrn = lambda c,rs: any(c>=r[0] and c<r[1] for r in rs)
chksz = lambda g: (gm:=g.meta)['bbw']==w and gm['bbh']==h

# Helper function to find clusters in fonts given a gap (sep[aration])
def font_clusters(glyphs, sep=1):
	cpv = (codepoints:= [g.cp() for g in glyphs])[0]
	cls, cl = [], [cpv]
	for c in codepoints[1:]:
		if c-cpv > sep:
			cls.append(cl)
			cl = [c]
		else:
			cl.append(c)
		cpv = c
	return cls

# Argument parsing
argparser = argparse.ArgumentParser(description="Convert BDF to wscons (BMH) compatible fonts")
argparser.add_argument('fonts', metavar='font', nargs='+', help="BDF font file")
argparser.add_argument('-v', '--verbose', action='store_true')
argparser.add_argument('-o', '--output', default='custom')
argparser.add_argument('--width', default=6)
argparser.add_argument('--height', default=12)
args = argparser.parse_args()
w, h, fontname = args.width, args.height, args.output

# Load fonts
fonts = [bdfparser.Font(f) for f in args.fonts]
glyphs = {c:g for f in fonts[::-1] for g in f.iterglyphs() if chkrn((c:=g.cp()),ranges) and chksz(g)}

# Prepare file handlers
open(f'{fontname}.h','w'); flf = open(f'{fontname}.h','a')
open('wsfont_extra.c','w'); flc = open('wsfont_extra.c','a')
open('char_list.txt','w'); fll = open('char_list.txt','a')

# Font preamble
print(f"""static u_char {fontname}_data[];

struct wsdisplay_font {fontname} = {{
		.name   = "{fontname}",
		.index    = 0,
		.firstchar  = ' ',
		.numchars = 65536 - ' ',
		.encoding = WSDISPLAY_FONTENC_UC8,
		.fontwidth  = {w},
		.fontheight = {h},
		.stride   = {(w+7)//8},
		.bitorder = WSDISPLAY_FONTORDER_L2R,
		.byteorder  = WSDISPLAY_FONTORDER_L2R,
		.cookie   = NULL,
		.data   = {fontname}_data
}};

static u_char {fontname}_data[] = {{""", file=flf)

# Main routine
odd,n = 0,0x20
hh = lambda n: hex(n).upper().replace('X','x')
for r in ranges:
	for j in range(*r):
		n +=1
		if j not in glyphs:
			print('\t'+'0x00,'*h, file=flf)
			odd +=1
			continue
		g = glyphs[j]
		print(f"\t/* {(gm:=g.meta)['glyphname']} */", file=flf)
		print(f"{g.chr()}\t0x{g.cp():X}\t{gm['glyphname']}\t{g.font.headers['fontname']}", file=fll)

		gh = gm['hexdata']
		#### Fixes to fonts/ranges/characters may be performed here e.g. the following commented example
		"""
		if j<0x200 and "spleen" in g.font.headers['fontname']:
			if any(f'SMALL LETTER {c}' in gm['glyphname'] for c in 'Y'): gh.pop(4); gh=['00']+gh
			elif gh[0]=='00' or 'LETTER' in gm['glyphname']: gh=['00']+gh[:-1]; #print("fixing ",g.chr())
			else: print(f"not fixing {g.chr()}")
		"""
		if(len(gh)!=h): print("failed at",g,f"{gh =}"); exit()
		for d in gh: print(f"\t0x{d},\t{pbmp(int(d,16))}", file=flf)

	if r == ranges[0]:
		rp = ranges[0]
		k = rp[1]
		print("\tif (font->encoding == WSDISPLAY_FONTENC_UC8) {", file=flc)
		print(f"\t\tif (c < {hh(rp[1])})\n\t\t\treturn (c);", file=flc)
	else:
		print(f"\t\telse if (c >= {hh(r[0])} && c < {hh(r[1])})\n\t\t\treturn (c{hh(-r[0] + k)});", file=flc)
		rp = r
		k += r[1]-r[0]

	if args.verbose:
		print('\n',r,odd,n)
		for font in fonts:
			print(sum(g.font == font for g in glyphs.values() if chkrn(g.cp(),[r])),"glyphs taken from",font.headers['fontname'])

print('};', file=flf)
print(f"""Done. You can find your font at {fontname},h
Copy the generated code in wsfont_extra.c to
/sys/dev/wsfont/wsfont.c inside the wsfont_map_unichar
function, right below
if (font->encoding == WSDISPLAY_FONTENC_ISO)
	return (c);
""")
